/* File:		mgTable.cpp
 * Author(s):	John W. Ratcliff	<jratcliff@verant.com>
 *				Michael Deardeuff	<mr__goodbytes@hotmail.com>
 * Descripton:
 *		Implements the StringTable class. For a better description, consult "sTable.h"
 * Notes:
 *		John W. Ratcliff made about 99% of this. All thanks go to him
 *		Michael Deardeuff just added IsIn() for use of CallFunctionByName
 */


#ifdef _MSC_VER // >= 1200					// For MSVC++ Only (6.0+ in comments)
#pragma warning(disable :4786)				// Disable all those pesky warnings you get
#endif // _MSC_VER							// from using the STL

#include <string.h>
#include <string>
#include <set>
#include "mgTable.h"

// Compares two strings, returns true if first is less than second (used by std::set)
bool CharPtrLess::operator()(const char *v1,const char *v2) const
{
	int v = strcmp(v1,v2);
	if ( v < 0 )
		return true;
	return false;
}


// Clean up.
StringTable::~StringTable(void)
{
	CharPtrSet::iterator i;
    for (i=mStrings.begin(); i!=mStrings.end(); i++)
    {
		char *str = (char *)(*i);
		delete str;
    }
}

/* Get():
 *	Searches the set, seeing if "str" is already in set.
 *	If so, returns a pointer to the string, If not,
 *	creates a new string, and returns a pointer to
 *	the newly allocated string;
 */
const char * StringTable::Get(const char *str)
{
    CharPtrSet::iterator found;

	// Search the set for str
    found = mStrings.find( str );

	// if found return pointer to the original string
    if ( found != mStrings.end() )
		return (*found);

	// create a new string identical to "str"
    int lngth = strlen(str);
    char *mem = new char[lngth+1];
    strcpy(mem,str);
    
	// insert new string into the set, and return a pointer
	mStrings.insert( mem );    
	return mem;
}

/* IsIn():
 *	Searches the set for occurance of "str". Returns True
 *	if found, false if not
 */
bool StringTable::IsIn(const char *str)
{
	CharPtrSet::iterator found;
	found = mStrings.find( str );
	if (found != mStrings.end())
		return true;
	return false;
}

const char * StringTable::Get(const String &str)
{
	return Get( str.c_str() );
}
